@@ -584,21 +584,21 @@ def add_run_args(parser, add_workflow_argument=True):
 
     # filescheme
 
-    fs_args = parser.add_argument_group("filescheme arguments")
-    fs_args.add_argument(
-        "-f",
-        "--filescheme",
-        choices=["local", "ssh"],
-        default="local",
-        help="Choose the filescheme required to retrieve the output files where your engine is located. "
-        "By selecting SSH, Janis will SCP the files using the --filescheme-ssh-binding SSH shortcut.",
-    )
-
-    fs_args.add_argument(
-        "--filescheme-ssh-binding",
-        help="Only valid if you've selected the ssh filescheme. "
-        "(eg: scp cluster:/path/to/output local/output/dir)",
-    )
+    # fs_args = parser.add_argument_group("filescheme arguments")
+    # fs_args.add_argument(
+    #     "-f",
+    #     "--filescheme",
+    #     choices=["local", "ssh"],
+    #     default="local",
+    #     help="Choose the filescheme required to retrieve the output files where your engine is located. "
+    #     "By selecting SSH, Janis will SCP the files using the --filescheme-ssh-binding SSH shortcut.",
+    # )
+    #
+    # fs_args.add_argument(
+    #     "--filescheme-ssh-binding",
+    #     help="Only valid if you've selected the ssh filescheme. "
+    #     "(eg: scp cluster:/path/to/output local/output/dir)",
+    # )
 
     validation_args = parser.add_argument_group("validation arguments")
 
@@ -681,6 +681,8 @@ def check_logger_args(args):
         level = LogLevel.from_str(args.logLevel)
 
     Logger.set_console_level(level)
+    if level > LogLevel.DEBUG:
+        Logger.set_write_level(level)
 
 
 def add_init_args(args):
@@ -872,13 +874,13 @@ def do_run(args):
         validation_reqs=validation_reqs,
         batchrun_reqs=batchrun_reqs,
         engine=args.engine,
-        filescheme=args.filescheme,
+        # filescheme=args.filescheme,
         hints=hints,
         output_dir=args.output_dir,
         execution_dir=args.execution_dir,
         inputs=inputs,
         required_inputs=required_inputs,
-        filescheme_ssh_binding=args.filescheme_ssh_binding,
+        # filescheme_ssh_binding=args.filescheme_ssh_binding,
         cromwell_url=args.cromwell_url,
         watch=args.progress,
         max_cores=args.max_cores,
@@ -954,15 +956,6 @@ def do_inputs(args):
         print(outs, file=sys.stdout)
 
 
-def do_environment(args):
-    method = args.method
-
-    if method == "list":
-        return print(", ".join(ConfigManager.manager().environmentDB.get_env_ids()))
-
-    raise NotImplementedError(f"No implementation for '{method}' yet")
-
-
 def do_query(args):
     status = None
     if args.status:
