@@ -6,19 +6,20 @@ import signal
 import subprocess
 import sys
 import threading
+import math
 from datetime import datetime
 from urllib import request, parse
 
 from glob import glob
 from typing import Optional, List, Tuple, Union
 
+from janis_assistant.data.models.run import RunModel
 from janis_core import LogLevel
 from janis_core.utils import first_value
 from janis_core.utils.logger import Logger
 
 from janis_assistant.__meta__ import ISSUE_URL
 from janis_assistant.data.models.outputs import WorkflowOutputModel
-from janis_assistant.data.models.workflow import WorkflowModel
 from janis_assistant.engines.cromwell.cromwellmetadata import (
     cromwell_status_to_status,
     CromwellMetadata,
@@ -54,6 +55,7 @@ class Cromwell(Engine):
         self._logger = None
         self._process = None
         self._timer_thread = None
+        self._start_time = DateUtil.now()
 
     def __init__(
         self,
@@ -96,6 +98,7 @@ class Cromwell(Engine):
         # Last contacted is used to determine
         self.last_contacted = None
         self.timeout = 10  # minutes
+        self._start_time = None
 
         self.connectionerrorcount = 0
         self.should_stop = False
@@ -136,7 +139,7 @@ class Cromwell(Engine):
             return True
 
         except Exception as e:
-            Logger.warn("Couldn't connect to Cromwell ({self.host}): " + str(e))
+            Logger.warn(f"Couldn't connect to Cromwell ({self.host}): {repr(e)}")
             return False
 
     def something_has_happened_to_cromwell(self, rc):
@@ -147,8 +150,14 @@ class Cromwell(Engine):
         for eid, callbacks in self.progress_callbacks.items():
             for cb in callbacks:
                 cb(
-                    WorkflowModel(
-                        engine_wid=eid, status=TaskStatus.SUSPENDED, error=message
+                    RunModel(
+                        id_=RunModel.DEFAULT_ID,
+                        submission_id=None,
+                        engine_id=eid,
+                        status=TaskStatus.SUSPENDED,
+                        error=message,
+                        execution_dir=None,
+                        name=None,
                     )
                 )
 
@@ -157,12 +166,33 @@ class Cromwell(Engine):
 
         self.stop_engine()
 
+    def get_poll_interval(self):
+        """
+        Get poll interval in SECONDS
+        """
+
+        max_poll = 60  # seconds
+        min_poll = 5  # seconds
+        falloff = 30 * 60  # seconds
+
+        # https://www.desmos.com/calculator/f5lmjtctfg
+        # ~ (max_poll - min_poll)/2 * (tanh(x/falloff - pi) + 1) + min_poll
+        #
+        # It's just a tanh graph that's been rebiased tanh(x-pi) + 1
+        # and then scaled
+
+        x = (DateUtil.now() - (self._start_time or DateUtil.now())).total_seconds()
+        return (max_poll - min_poll) / 2 * (
+            math.tanh(math.pi * (2 * x / falloff - 1)) + 1
+        ) + min_poll
+
     def start_engine(self, additional_cromwell_options: List[str] = None):
 
         from janis_assistant.management.configuration import JanisConfiguration
 
         jc = JanisConfiguration.manager()
 
+        self._start_time = DateUtil.now()
         self.timeout = jc.cromwell.timeout or 10
 
         if self.test_connection():
@@ -189,7 +219,7 @@ class Cromwell(Engine):
         Logger.log("Finding cromwell jar")
         cromwell_loc = self.resolve_jar(self.cromwelljar)
 
-        Logger.info(f"Starting cromwell ({os.path.basename(cromwell_loc)})...")
+        Logger.info(f"Starting cromwell ({cromwell_loc})...")
         cmd = ["java", "-DLOG_MODE=standard"]
 
         if jc.cromwell and jc.cromwell.memory:
@@ -197,8 +227,8 @@ class Cromwell(Engine):
                 [f"-Xmx{jc.cromwell.memory}M", f"-Xms{max(jc.cromwell.memory//2, 1)}M"]
             )
 
-        if Logger.CONSOLE_LEVEL == LogLevel.VERBOSE:
-            cmd.append("-DLOG_LEVEL=DEBUG")
+        # if Logger.CONSOLE_LEVEL == LogLevel.VERBOSE:
+        #     cmd.append("-DLOG_LEVEL=DEBUG")
 
         if additional_cromwell_options:
             cmd.extend(additional_cromwell_options)
@@ -228,7 +258,11 @@ class Cromwell(Engine):
         )
 
         self._logfp = open(self.logfile, "a+")
-        Logger.info("Will log to file" if bool(self._logfp) else "Will NOT log to file")
+        Logger.info(
+            f"Will log Cromwell output to the file: {self.logfile}"
+            if bool(self._logfp)
+            else "Janis is NOT logging Cromwell output to a file"
+        )
 
         for c in iter(
             self._process.stdout.readline, "b"
@@ -407,13 +441,30 @@ class Cromwell(Engine):
             return
 
         for engine_id_to_poll in self.progress_callbacks:
-            meta = self.metadata(engine_id_to_poll)
-            if meta:
+            try:
+                meta = self.metadata(engine_id_to_poll)
+                if meta:
+                    for callback in self.progress_callbacks[engine_id_to_poll]:
+                        callback(meta)
+
+            except Exception as e:
+                Logger.critical(
+                    f"Received a critical error ({repr(e)}) when getting metadata for "
+                    f"Cromwell task {engine_id_to_poll}, hence terminating task with status=SUSPENDED"
+                )
+                meta = RunModel(
+                    id_=None,
+                    submission_id=None,
+                    status=TaskStatus.SUSPENDED,
+                    engine_id=engine_id_to_poll,
+                    name=None,
+                    execution_dir=None,
+                )
                 for callback in self.progress_callbacks[engine_id_to_poll]:
                     callback(meta)
 
         # call timer again
-        time = 6
+        time = self.get_poll_interval()
         threading.Timer(time, self.poll_metadata).start()
 
     @staticmethod
@@ -673,15 +724,23 @@ class Cromwell(Engine):
                     "last_updated_threshold"
                 )  # idk, pick a number
                 return None
-            if self.connectionerrorcount > 50:
+            if self.connectionerrorcount > 5:
                 raise e
             else:
                 Logger.warn("Error connecting to cromwell instance: " + str(e))
             return None
 
-    def metadata(self, identifier, expand_subworkflows=True) -> Optional[WorkflowModel]:
+    def metadata(self, identifier, expand_subworkflows=True) -> Optional[RunModel]:
         if self.error_message:
-            return WorkflowModel(error=self.error_message, status=TaskStatus.FAILED)
+            return RunModel(
+                id_=RunModel.DEFAULT_ID,
+                submission_id=None,
+                error=self.error_message,
+                status=TaskStatus.FAILED,
+                engine_id=identifier,
+                execution_dir=None,
+                name=None,
+            )
 
         raw = self.raw_metadata(identifier, expand_subworkflows=expand_subworkflows)
         return raw.standard() if raw else raw
