@@ -5,6 +5,7 @@ import subprocess
 from typing import Dict, Any
 
 from janis_core import LogLevel
+from janis_core.types.data_types import is_python_primitive
 from janis_core.utils.logger import Logger
 from janis_assistant.data.models.outputs import WorkflowOutputModel
 from janis_assistant.data.models.run import RunModel
@@ -20,8 +21,8 @@ class CWLToolLogger(ProcessLogger):
 
     statusupdateregex = re.compile("INFO \[(.*)\] (.+)$")
 
-    def __init__(self, wid: str, process, logfp, metadata_callback, exit_function=None):
-        self.wid = wid
+    def __init__(self, sid: str, process, logfp, metadata_callback, exit_function=None):
+        self.sid = sid
 
         self.error = None
         self.metadata_callback = metadata_callback
@@ -173,7 +174,7 @@ class CWLToolLogger(ProcessLogger):
 
         job = RunJobModel(
             submission_id=None,
-            run_id=self.wid,
+            run_id=self.sid,
             id_=jid,
             parent=parentid,
             name=stepname,
@@ -243,6 +244,7 @@ class CWLTool(Engine):
 
         # we're going to abort!
         if self.process_id:
+            Logger.info("Received stop_engine request for CWLTool")
             try:
                 import signal
 
@@ -286,7 +288,7 @@ class CWLTool(Engine):
 
         updates = {}
 
-        if isinstance(out, str):
+        if is_python_primitive(out):
             updates[key] = WorkflowOutputModel(
                 submission_id=None,
                 run_id=run_id,
@@ -363,7 +365,7 @@ class CWLTool(Engine):
             engine_id=identifier,
             execution_dir=None,
             submission_id=None,
-            # name=identifier,
+            name=identifier,
             status=self.taskmeta.get("status"),
             # start=self.taskmeta.get("start"),
             # finish=self.taskmeta.get("finish"),
@@ -436,12 +438,12 @@ class CWLTool(Engine):
         if logger.error:
             self.taskmeta["error"] = logger.error
 
-        for callback in self.progress_callbacks.get(logger.wid, []):
-            callback(self.metadata(logger.wid))
+        for callback in self.progress_callbacks.get(logger.sid, []):
+            callback(self.metadata(logger.sid))
 
     def task_did_update(self, logger: CWLToolLogger, job: RunJobModel):
-        Logger.info(f"Updated task {job.id_} with status={job.status}")
+        Logger.debug(f"Updated task {job.id_} with status={job.status}")
         self.taskmeta["jobs"][job.id_] = job
 
-        for callback in self.progress_callbacks.get(logger.wid, []):
-            callback(self.metadata(logger.wid))
+        for callback in self.progress_callbacks.get(logger.sid, []):
+            callback(self.metadata(logger.sid))
