@@ -2,14 +2,14 @@ import os
 import sqlite3
 from datetime import datetime
 from shutil import rmtree
-from typing import Dict, Optional, Union
+from typing import Dict, Optional, Union, List
 from contextlib import contextmanager
 
+from janis_assistant.engines import Engine
 from janis_core import Workflow, Logger, Tool
 
-from janis_assistant.data.models.workflow import WorkflowModel
+from janis_assistant.data.models.run import RunModel
 from janis_assistant.data.providers.janisdbprovider import TasksDbProvider, TaskRow
-from janis_assistant.environments.environment import Environment
 from janis_assistant.management.configuration import JanisConfiguration
 from janis_assistant.management.workflowmanager import WorkflowManager
 from janis_assistant.utils import generate_new_id, fully_qualify_filename
@@ -43,24 +43,18 @@ class ConfigManager:
 
         self._connection: Optional[sqlite3.Connection] = None
         self._taskDB: Optional[TasksDbProvider] = None
-        # self.environmentDB = EnvironmentDbProvider(self.connection, self.cursor)
-        # self.engineDB = EngineDbProvider(self.connection, self.cursor)
-        # self.fileschemeDB = FileschemeDbProvider(self.connection, self.cursor)
-
-        # if self.is_new:
-        #     self.insert_default_environments()
 
     def get_lazy_db_connection(self):
         if self._taskDB is None:
             self._connection = self.db_connection()
 
-            self._taskDB = TasksDbProvider(self._connection)
+            self._taskDB = TasksDbProvider(self._connection, readonly=self.readonly)
 
         return self._taskDB
 
     @contextmanager
     def with_cursor(self):
-        yield self.get_lazy_db_connection().db.cursor()
+        yield self.get_lazy_db_connection()._db.cursor()
 
     def db_connection(self):
         config = JanisConfiguration.manager()
@@ -91,7 +85,7 @@ class ConfigManager:
         tm = WorkflowManager.from_path_with_submission_id(
             task.outputdir, task.submission_id
         )
-        tm.remove_exec_dir()
+        tm.remove_exec_dir_if_required()
         tm.database.close()
 
         if not keep_output and os.path.exists(task.outputdir):
@@ -195,7 +189,7 @@ class ConfigManager:
         tool: Tool,
         output_dir: str,
         execution_dir: str,
-        environment: Environment,
+        engine: Engine,
         hints: Dict[str, str],
         validation_requirements: Optional[ValidationRequirements],
         batchrun_requirements: Optional[BatchRunRequirements],
@@ -218,7 +212,7 @@ class ConfigManager:
             tool=tool,
             output_dir=output_dir,
             execution_dir=execution_dir,
-            environment=environment,
+            engine=engine,
             hints=hints,
             inputs_dict=inputs_dict,
             validation_requirements=validation_requirements,
@@ -236,6 +230,28 @@ class ConfigManager:
             **kwargs,
         )
 
+    def get_row_for_submission_id_or_path(self, submission_id) -> TaskRow:
+
+        potential_submission = self.get_lazy_db_connection().get_by_id(submission_id)
+        if potential_submission:
+            return potential_submission
+
+        expanded_path = fully_qualify_filename(submission_id)
+        if os.path.exists(expanded_path):
+            (execpath, sid) = WorkflowManager.from_path_get_latest_submission_id(
+                expanded_path
+            )
+            return TaskRow(
+                execution_dir=expanded_path,
+                submission_id=sid,
+                output_dir=None,
+                timestamp=None,
+            )
+
+        raise Exception(
+            f"Couldn't find task with id='{submission_id}', and no directory was found."
+        )
+
     def from_submission_id_or_path(self, submission_id, readonly=False):
 
         self.readonly = readonly
@@ -252,7 +268,7 @@ class ConfigManager:
 
         expanded_path = fully_qualify_filename(submission_id)
         if os.path.exists(expanded_path):
-            return WorkflowManager.from_path_get_latest(
+            return WorkflowManager.from_path_get_latest_manager(
                 expanded_path, readonly=readonly
             )
 
@@ -260,27 +276,27 @@ class ConfigManager:
             f"Couldn't find task with id='{submission_id}', and no directory was found "
         )
 
-    def query_tasks(self, status, name) -> Dict[str, WorkflowModel]:
+    def query_tasks(self, status, name) -> Dict[str, RunModel]:
 
-        rows: [TaskRow] = self.get_lazy_db_connection().get_all_tasks()
+        rows: List[TaskRow] = self.get_lazy_db_connection().get_all_tasks()
 
         failed = []
         relevant = {}
 
         for row in rows:
-            if not os.path.exists(row.outputdir):
+            if not os.path.exists(row.execution_dir):
                 failed.append(row.submission_id)
                 continue
             try:
                 metadb = WorkflowManager.has(
-                    row.outputdir,
+                    row.execution_dir,
                     submission_id=row.submission_id,
                     name=name,
                     status=status,
                 )
                 if metadb:
-                    model = metadb.to_model()
-                    model.outdir = row.outputdir
+                    model = metadb
+                    model.outdir = row.output_dir
                     relevant[row.submission_id] = model
             except Exception as e:
                 Logger.critical(f"Couldn't check workflow '{row.submission_id}': {e}")
