@@ -130,6 +130,7 @@ class JanisConfiguration(NoAttributeErrors):
             MySqlDbName = "mysql_dbname"
             UseManagedMySqlInstance = "should_manage_mysql"
             UseDatabase = "use_database"
+            AdditionalParameters = "additional_params"  # string or list of strings
 
         def __init__(self, d: dict, default: dict):
             d = d if d else {}
@@ -173,6 +174,10 @@ class JanisConfiguration(NoAttributeErrors):
                 d, self.Keys.UseDatabase, default
             )
 
+            self.additional_params = JanisConfiguration.get_value_for_key(
+                d, self.Keys.AdditionalParameters, default
+            )
+
         def get_database_config_helper(self):
             existing_config = None
             if self.mysql_url:
@@ -437,7 +442,7 @@ class JanisConfiguration(NoAttributeErrors):
     @staticmethod
     def get_value_for_key(d, key, default):
         val = d.get(key)
-        if not val:
+        if val is None:
             return default.get(key) if default else None
 
         Logger.log(f"Got value '{val}' for key '{key}'")
